/*
Copyright (c) 2010 Christian NineBerry Schwarz <http://www.nineberry.de>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
function Schatzkarte(){
  this.plz_manager = new PLZManager();
  this.gemeinde_manager = new GemeindeManager();
  this.wahlkreis_manager = new WahlkreisManager();
  this.stadtteil_manager = new StadtteilManager();
  this.strassen_manager = new StrassenManager();
  this.plz = '';
  this.gemeinde = 0;
  this.strasse = 0;
  this.hausnummer = 0;
  this.stadtteil = 0;
  this.strassen_plz = '';

  this.plz = function(){
    for(var i=0; i < arguments.length - 2; i+=2){
	  this.plz_manager.add_entry(arguments[i], arguments[i+1]);
	}
  }

  this.wahlkreise = function(){
    for(var i=0; i < arguments.length; i++){
	  this.wahlkreis_manager.add_entry(arguments[i]);
	}
  }

  this.gemeinden = function(){
    for(var i=0; i < arguments.length - 3; i+=3){
	  this.gemeinde_manager.add_entry(arguments[i], arguments[i+1], arguments[i+2]);
	}
  }

  this.stadtteile = function(){
    for(var i=0; i < arguments.length - 4; i+=4){
	  this.stadtteil_manager.add_entry(arguments[i], arguments[i+1], arguments[i+2], arguments[i+3]);
	}
  }

  this.strassen = function(){
    var gemeindeid = arguments[0];
	for(var i=1; i < arguments.length - 6; i+=6){
	  this.strassen_manager.add_entry(gemeindeid, arguments[i], arguments[i+1], arguments[i+2],
	    arguments[i+3],  arguments[i+4],  arguments[i+5]);
	}
  }

  this.plz_change = function(input){
    if(this.plz != input.value){
	  this.plz= input.value;
	  this.update_display();
	}
  }

  this.gemeinde_change = function(input){
    if(this.gemeinde != input.value){
	  this.gemeinde = input.value;
	  this.update_display();
	 }
  }

  this.strasse_change = function(input){
    if(this.strasse != input.value){
	  this.strasse= input.value;
	  this.update_display();
	}
  }

  this.stadtteil_change = function(input){
    if(this.stadtteil != input.value){
	  this.stadtteil= input.value;
	  this.update_display();
	}
  }

  this.hausnummer_change = function(input){
    var hn = parseInt(input.value);

	if(this.hausnummer != hn){
	  this.hausnummer= hn;
	  this.update_display();
	}
  }

  this.update_display = function(){
    // PLZ
	sk_show_element('sk_plz');
	var gemeinden = this.plz_manager.find_entry(this.plz);
	var elem_plz_status = sk_find_element('sk_plz_status');

	if (gemeinden.length > 0){
	  elem_plz_status.innerHTML = 'PLZ <strong>' + this.plz + '</strong> gefunden';
	  sk_show_element('sk_gemeinde');

	  this.populate_gemeinden(gemeinden);

	  // Gemeinde
	  var elem_gemeinde_status = sk_find_element('sk_gemeinde_status');
	  if(this.gemeinde != 0){
		var gemeinde = this.gemeinde_manager.find_entry(this.gemeinde);
		elem_gemeinde_status.innerHTML = 'Ort <strong>' + gemeinde.name + '</strong> ausgewählt';

		// Haben wir schon endgültigen Wahlkreis?
		if(gemeinde.wahlkreis != 0){
		    sk_hide_element('sk_stadtteil');
		    sk_hide_element('sk_strasse');
		    sk_hide_element('sk_hausnummer');
			this.strasse = 0;
			this.stadtteil = 0;
			this.hausnummer = 0;
			this.strassen_plz = '';
			sk_clear_input('sk_hausnummer_input');

			sk_show_element('sk_ergebnis');
			this.populate_ergebnis(gemeinde.wahlkreis);
		}
		else{
		    sk_show_element('sk_stadtteil');

			// Stadtteile laden und ausgeben
			this.populate_stadtteile(this.gemeinde);

			var elem_stadtteil_status = sk_find_element('sk_stadtteil_status');
			if(this.stadtteil != 0){
			    var stadtteil = this.stadtteil_manager.find_entry(this.gemeinde, this.stadtteil);
				elem_stadtteil_status.innerHTML = 'Stadtteil <strong>' + stadtteil.name + '</strong> ausgewählt';

				sk_hide_element('sk_hausnummer');
				this.hausnummer = 0;
				sk_clear_input('sk_hausnummer_input');

				sk_show_element('sk_ergebnis');

				var wahlkreis = this.stadtteil_manager.find_wahlkreis(this.gemeinde, this.stadtteil);
				this.populate_ergebnis(wahlkreis);
			}
			else{
				elem_stadtteil_status.innerHTML = 'Kein Stadtteil ausgewählt';
				sk_hide_element('sk_hausnummer');
				this.hausnummer = 0;
				sk_clear_input('sk_hausnummer_input');
				sk_hide_element('sk_ergebnis');
			}
		}
	  }
	  else{
	      elem_gemeinde_status.innerHTML = 'Kein Ort ausgewählt';
		  this.strasse = 0;

		  sk_hide_element('sk_stadtteil');
		  sk_hide_element('sk_strasse');
		  sk_hide_element('sk_hausnummer');
		  this.strasse = 0;
		  this.hausnummer = 0;
		  this.strassen_plz = '';
		  this.stadtteil = 0;
		  sk_clear_input('sk_hausnummer_input');
		  sk_hide_element('sk_ergebnis');
	  }
	}
	else{
	  elem_plz_status.innerHTML = 'Keine gültige PLZ eingegeben';
	  sk_hide_element('sk_gemeinde');
	  sk_hide_element('sk_strasse');
	  sk_hide_element('sk_hausnummer');
	  sk_hide_element('sk_stadtteil');
	  this.gemeinde = 0;
	  this.strasse = 0;
	  this.hausnummer = 0;
	  this.stadtteil = 0;
	  this.strassen_plz = '';
	  sk_clear_input('sk_hausnummer_input');
	  sk_hide_element('sk_ergebnis');
	}
  }

  this.populate_gemeinden = function(gemeinden){
    var elem_select = sk_find_element('sk_gemeinde_input');

	while(elem_select.length > 0){
	  elem_select.remove(0);
	}

	// Objekte laden
	gemeindeObjs = new Array();
	for (var i in gemeinden){

	  var gemeinde = this.gemeinde_manager.find_entry(gemeinden[i]);
	  gemeindeObjs.push(gemeinde);
	}

	// "Bitte auswählen" einfügen
	if(gemeindeObjs.length > 1){
	  var elem_new_opt = document.createElement('option');
	  elem_new_opt.text = ' [ Ort wählen ] ';
	  elem_new_opt.value = 0;
	  elem_select.options[elem_select.options.length] = elem_new_opt;
	}

	// In Select einfügen
	var anySelected = false;
	for (var gemeinde in gemeindeObjs){
	  var gemeindeObj = gemeindeObjs[gemeinde];

	  var elem_new_opt = document.createElement('option');
	  elem_new_opt.text = gemeindeObj.name;
	  elem_new_opt.value = gemeindeObj.id;

	  if (this.gemeinde == gemeindeObj.id){
	    elem_new_opt.selected=true;
		anySelected = true;
	  }

	  elem_select.options[elem_select.options.length] = elem_new_opt;
	}

	// Falls keins ausgewählt, dann erstes auswählen
	if (!anySelected){
	  elem_select.selectedIndex = 0;
	  this.gemeinde_change(elem_select);
	}
  }

  this.populate_strassen = function(gemeinde, plz){
    var elem_select = sk_find_element('sk_strasse_input');

	if(this.strassen_plz == plz){
	  // Gleiche PLZ, nicht neu laden,
	  // Prozessor schonen
	  return;
	}

	this.strassen_plz = plz;

	while(elem_select.length > 0){
	  elem_select.remove(0);
	}

	// Hole alle Straßen passend zur Gemeinde und PLZ
	var strassen = this.strassen_manager.find_strassen(gemeinde, plz);

	// "Bitte auswählen" einfügen
	if(strassen.length > 1){
	  var elem_new_opt = document.createElement('option');
	  elem_new_opt.text = ' [ Straße wählen ] ';
	  elem_new_opt.value = 0;
	  elem_select.options[elem_select.options.length] = elem_new_opt;
	}


	var anySelected = false;
	for (var i in strassen){
	  var strassenObj = strassen[i];

	  var elem_new_opt = document.createElement('option');
	  elem_new_opt.text = strassenObj.name;
	  elem_new_opt.value = strassenObj.id;

	  if (this.strasse == strassenObj.id){
	    elem_new_opt.selected=true;
		anySelected = true;
	  }

	  elem_select.options[elem_select.options.length] = elem_new_opt;
	}

	if (!anySelected){
	  elem_select.selectedIndex = 0;
	  this.strasse_change(elem_select);
	}
  }

  this.populate_stadtteile = function(gemeinde){
    var elem_select = sk_find_element('sk_stadtteil_input');

	while(elem_select.length > 0){
	  elem_select.remove(0);
	}

	// Hole alle Straßen passend zur Gemeinde und PLZ
	var stadtteile = this.stadtteil_manager.find_stadtteile(gemeinde);

	// "Bitte auswählen" einfügen
	if(stadtteile.length > 1){
	  var elem_new_opt = document.createElement('option');
	  elem_new_opt.text = ' [ Stadtteil wählen ] ';
	  elem_new_opt.value = 0;
	  elem_select.options[elem_select.options.length] = elem_new_opt;
	}


	var anySelected = false;
	for (var i in stadtteile){
	  var stadtteilObj = stadtteile[i];

	  var elem_new_opt = document.createElement('option');
	  elem_new_opt.text = stadtteilObj.name;
	  elem_new_opt.value = stadtteilObj.id;

	  if (this.stadtteil == stadtteilObj.id){
	    elem_new_opt.selected=true;
		anySelected = true;
	  }

	  elem_select.options[elem_select.options.length] = elem_new_opt;
	}

	if (!anySelected){
	  elem_select.selectedIndex = 0;
	  this.stadtteil_change(elem_select);
	}
  }

  this.populate_ergebnis = function(wahlkreis){

	var wahlkreisObj = this.wahlkreis_manager.find_entry(wahlkreis);

	var elem_ergebnis_name = sk_find_element('sk_ergebnis_name');
	elem_ergebnis_name.innerHTML = wahlkreisObj.name;

    var elem_ergebnis_nummer = sk_find_element('sk_ergebnis_nummer');
	elem_ergebnis_nummer.innerHTML = wahlkreis;

    var elem_ergebnis_url = sk_find_element('sk_ergebnis_url');
	elem_ergebnis_url.href = 'http://ltw.piratenpartei-bw.de/WK/' + wahlkreis;

    var address = this.plz + ' ';

	var gemeinde = this.gemeinde_manager.find_entry(this.gemeinde);
	address = address + gemeinde.name + ' ';

	if(this.stadtteil){
	  address = address + this.stadtteil_manager.find_entry(this.gemeinde, this.stadtteil).name + ' ';
	}

	if(this.strasse){
	  address = address + this.strassen_manager.find_strassen_name(this.gemeinde, this.strasse) + ' ';
	}

	if((this.hausnummer != 0) && (!isNaN(this.hausnummer))){
	  address = address + this.hausnummer + ' ';
	}

	var elem_ergebnis_address = sk_find_element('sk_ergebnis_address');
	elem_ergebnis_address.innerHTML = address;
  }
}

function PLZManager(){
  this.entries = new Object();

  this.add_entry = function(plz, gemeindeid){
    if(!this.entries[plz]){
	  this.entries[plz] = new Array();
	}

	this.entries[plz].push(gemeindeid);
  }

  this.find_entry = function(plz){
	if(this.entries[plz]){
	  return this.entries[plz];
	}
	else{
	  return new Array;
	}
  }
}

function Gemeinde(id, wahlkreis, name){
  this.id = id;
  this.wahlkreis = wahlkreis;
  this.name = name;
}

function GemeindeManager(){
  this.entries = new Object();

  this.add_entry = function(gemeindeid, wahlkreis, name){
	var gemeinde = new Gemeinde(gemeindeid, wahlkreis, name);
	this.entries[gemeindeid] = gemeinde;
  }

  this.find_entry = function(gemeindeid){
	  return this.entries[gemeindeid];
  }
}

function Wahlkreis(id, name){
  this.id = id;
  this.name = name;
}

function WahlkreisManager(){
  this.entries = new Array();

  this.add_entry = function(name){
	var id = this.entries.length + 1;
	var wahlkreis = new Wahlkreis(id, name);
	this.entries.push(wahlkreis);
  }

  this.find_entry = function(wahlkreisid){
	  return this.entries[wahlkreisid-1];
  }
}

function Stadtteil(gemeindeid, id, wahlkreis, name){
  this.gemeindeid = gemeindeid;
  this.id = id;
  this.wahlkreis = wahlkreis;
  this.name = name;
}

function StadtteilManager(){
  this.entries = new Object();

  this.add_entry = function(gemeindeid, id, wahlkreis, name){
	var stadtteil = new Stadtteil(gemeindeid, id, wahlkreis, name);
	this.entries[gemeindeid + '_' + id] = stadtteil;
  }

  this.find_entry = function(gemeindeid, id){
	  return this.entries[gemeindeid + '_' + id];
  }

  this.find_wahlkreis = function(gemeindeid, id){
	  var stadtteil = this.entries[gemeindeid + '_' + id];
	  return stadtteil.wahlkreis;
  }

  this.find_stadtteile = function(gemeinde){
	var res = new Array();

    for(var i in this.entries){
	  var st = this.entries[i];

	  if (st.gemeindeid == gemeinde){
		  res.push(st);
	  }
	}
	return res;
  }


}

function Strasse(strassenid, name, startno, endno, stadtteilid, plz){
  this.id = strassenid;
  this.name = name;
  this.startno = startno;
  this.endno = endno;
  this.stadtteilid = stadtteilid;
  this.plz = plz;
}

function GemeindeStrassen(gemeindeid){
  this.entries = new Array();
  this.gemeindeid = gemeindeid;
  this.loaded = false;

  this.add_entry = function(strassenid, name, startno, endno, stadtteilid, plz){
    var str = new Strasse(strassenid, name, startno, endno, stadtteilid, plz);
	this.entries.push(str);
  }

  this.find_strassen = function(plz){
	var res = new Array();
	var schonGefunden = new Object();

    for(var i in this.entries){
	  var str = this.entries[i];

	  // Vorerst alle Straßen verwenden, da PLZ-Zuordnung in unseren Datensätzen
	  // unvollständig
	  if(true){ //if (str.plz == plz){
		if(!schonGefunden[str.id]){
		  schonGefunden[str.id] = true;
		  res.push(str);
		}
	  }
	}
	return res;
  }

  this.find_strasse_stadtteil = function(strassenid){
    for(var i in this.entries){
	  var str = this.entries[i];

	  if (str.id == strassenid){
		if (str.startno == 0){
          // Unterscheidung nach Hausnummer nicht notwendig
		  return str.stadtteilid;
		}
		else{
          // Unterscheidung nach Hausnummer ist notwendig
		  return 0;
		 }
	  }
	}

	return 0;
  }

  this.find_hausnummer_stadtteil = function(strassenid, hausnummer){
    for(var i in this.entries){
	  var str = this.entries[i];

	  if (str.id == strassenid){
		if (str.startno == 0){
          // Unterscheidung nach Hausnummer nicht notwendig
		  return str.stadtteilid;
		}

		if(sk_isEven(hausnummer) && sk_isEven(str.startno)){
		  if((hausnummer >= str.startno) && (hausnummer <= str.endno)){
		    return str.stadtteilid;
		  }
		}

		if(!sk_isEven(hausnummer) && !sk_isEven(str.startno)){
		  if((hausnummer >= str.startno) && (hausnummer <= str.endno)){
		    return str.stadtteilid;
		  }
		}
	  }
	}

	return 0;
  }

  this.find_strassen_name = function(strassenid){
    for(var i in this.entries){
	  var str = this.entries[i];

	  if (str.id == strassenid){
		  return str.name;
	  }
	}

	return "[-]";
  }
}

function StrassenManager(){
  this.entries = new Object();

  this.load = function(gemeinde){
    if(!this.entries[gemeinde]){
	    this.entries[gemeinde] = new GemeindeStrassen(gemeinde);
	    var filename= 'data/strassen' + gemeinde + '.js';

		var headID = document.getElementsByTagName("head")[0];
		var newScript = document.createElement('script');
		newScript.type = 'text/javascript';
		newScript.src = filename;
		headID.appendChild(newScript);
		return false;
	}
	return this.entries[gemeinde].loaded;
  }

  this.start_loading = function(gemeinde){
  }

  this.end_loading = function(gemeinde){
    this.entries[gemeinde].loaded = true;
    schatzkarte.update_display();
  }

  this.add_entry = function(gemeindeid, strassenid, name, startno, endno, stadtteilid, plz){
	var strassen = this.entries[gemeindeid];
	strassen.add_entry(strassenid, name, startno, endno, stadtteilid, plz);
  }

  this.find_strassen = function(gemeindeid, plz){
	var strassen = this.entries[gemeindeid];
	return strassen.find_strassen(plz);
  }

  this.find_strasse_stadtteil = function(gemeindeid, strassenid){
	var strassen = this.entries[gemeindeid];
	return strassen.find_strasse_stadtteil(strassenid);
  }

  this.find_hausnummer_stadtteil = function(gemeindeid, strassenid, hausnummer){
	var strassen = this.entries[gemeindeid];
	return strassen.find_hausnummer_stadtteil(strassenid, hausnummer);
  }

  this.find_strassen_name = function(gemeindeid, strassenid){
	var strassen = this.entries[gemeindeid];
	return strassen.find_strassen_name(strassenid);
  }
}

var schatzkarte = new Schatzkarte();